/************************************************************************/
/* gtc.h - GPL Telescope Control Library Header				*/
/************************************************************************/
/*
 *  (c) 2004 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/

#ifndef _GTC_H_
#define _GTC_H_


#include <gtc/gtctypes.h>

/* library support functions */
/* NOTE: returned strings should NOT be freed by the application */
int gtc_library_get_version (char **version);

/* module loading/unloading */
/* NOTE: returned strings should NOT be freed by the application */
int gtc_module_load (GtcModule **gmp, char *path);
int gtc_module_get_name (GtcModule *gm, char **name);
int gtc_module_get_description (GtcModule *gm, char **description);
int gtc_module_get_version (GtcModule *gm, char **version);
int gtc_module_get_author (GtcModule *gm, char **author);
int gtc_module_get_path (GtcModule *gm, char **path);
int gtc_module_get_usecount (GtcModule *gm, int *usecount);
int gtc_module_unload (GtcModule **gmp);

/* scope connect/disconnect */
int gtc_scope_new (GtcScope **gsp, GtcModule *gm);
int gtc_scope_connect (GtcScope *gs, char *device);
int gtc_scope_close (GtcScope *gs);
int gtc_scope_free (GtcScope **gsp);

/* scope model/firmware information */
/* NOTE: returned strings should NOT be freed by the application, and */
/*       the string will be valid until the next gtc_scope_* API call */
int gtc_scope_get_model (GtcScope *gs, char **model);
int gtc_scope_get_firmware_version (GtcScope *gs, char **version);
int gtc_scope_get_firmware_date (GtcScope *gs, char **date);

/* scope date/time */
int gtc_scope_get_datetime (GtcScope *gs, GtcDate *gd, GtcTime *gt);
int gtc_scope_set_datetime (GtcScope *gs, GtcDate *gd, GtcTime *gt);
int gtc_scope_get_utc_offset (GtcScope *gs, double *offset);
int gtc_scope_set_utc_offset (GtcScope *gs, double offset);
int gtc_scope_get_sidereal_time (GtcScope *gs, GtcTime *gt);
int gtc_scope_set_sidereal_time (GtcScope *gs, GtcTime *gt);

/* scope mount/site/location */
int gtc_scope_autoalign (GtcScope *gs);
int gtc_scope_home (GtcScope *gs);
int gtc_scope_get_alignment (GtcScope *gs, int *alignment);
int gtc_scope_set_alignment (GtcScope *gs, int alignment);
int gtc_scope_get_longitude (GtcScope *gs, double *longitude);
int gtc_scope_set_longitude (GtcScope *gs, double longitude);
int gtc_scope_get_latitude (GtcScope *gs, double *latitude);
int gtc_scope_set_latitude (GtcScope *gs, double latitude);
int gtc_scope_get_current_site (GtcScope *gs, int *site);
int gtc_scope_set_current_site (GtcScope *gs, int site);

/* scope RA/DEC motion */
int gtc_scope_slew_east_start (GtcScope *gs);
int gtc_scope_slew_east_stop (GtcScope *gs);
int gtc_scope_slew_west_start (GtcScope *gs);
int gtc_scope_slew_west_stop (GtcScope *gs);
int gtc_scope_slew_north_start (GtcScope *gs);
int gtc_scope_slew_north_stop (GtcScope *gs);
int gtc_scope_slew_south_start (GtcScope *gs);
int gtc_scope_slew_south_stop (GtcScope *gs);
int gtc_scope_slew_stop (GtcScope *gs);

int gtc_scope_get_ra_slewrate (GtcScope *gs, double *degsec);
int gtc_scope_set_ra_slewrate (GtcScope *gs, double degsec); 
int gtc_scope_get_dec_slewrate (GtcScope *gs, double *degsec);
int gtc_scope_set_dec_slewrate (GtcScope *gs, double degsec); 

int gtc_scope_get_trackrate (GtcScope *gs, double *hertz);
int gtc_scope_set_trackrate (GtcScope *gs, double hertz);

int gtc_scope_get_guiderate (GtcScope *gs, double *arcsecsec);
int gtc_scope_set_guiderate (GtcScope *gs, double arcsecsec);

/* scope focus */
int gtc_scope_focus_in_start (GtcScope *gs);
int gtc_scope_focus_out_start (GtcScope *gs);
int gtc_scope_focus_stop (GtcScope *gs);
int gtc_scope_focus_get_speed (GtcScope *gs, double *speed);
int gtc_scope_focus_set_speed (GtcScope *gs, double speed);

/* scope reticle */
int gtc_scope_reticle_get_brightness (GtcScope *gs, double *brightness);
int gtc_scope_reticle_set_brightness (GtcScope *gs, double brightness);
int gtc_scope_reticle_get_flashrate (GtcScope *gs, double *rate);
int gtc_scope_reticle_set_flashrate (GtcScope *gs, double rate);
int gtc_scope_reticle_get_dutyrate (GtcScope *gs, double *rate);
int gtc_scope_reticle_set_dutyrate (GtcScope *gs, double rate);

/* scope field derotator */
int gtc_scope_derotator_enable (GtcScope *gs, int enable);

/* scope targeting/tracking */
int gtc_scope_get_current_ra (GtcScope *gs, GtcRa *ra);
int gtc_scope_get_current_dec (GtcScope *gs, GtcDec *dec);

int gtc_scope_get_target_ra (GtcScope *gs, GtcRa *ra);
int gtc_scope_set_target_ra (GtcScope *gs, GtcRa *ra);
int gtc_scope_get_target_dec (GtcScope *gs, GtcDec *dec);
int gtc_scope_set_target_dec (GtcScope *gs, GtcDec *dec);
int gtc_scope_goto_target (GtcScope *gs);

/* scope library objects */
int gtc_scope_object_set_target (GtcScope *gs, int catalog, long object);

/* scope Periodic Error Correction (PEC */
int gtc_scope_pec_ra_enable (GtcScope *gs, int enable);
int gtc_scope_pec_dec_enable (GtcScope *gs, int enable);

/* scope options */
int gtc_scope_option_list (GtcScope *gs, GtcOption **optionlist);
int gtc_scope_option_get_int (GtcScope *gs, char *name, int *value);
int gtc_scope_option_set_int (GtcScope *gs, char *name, int value);
int gtc_scope_option_get_boolean (GtcScope *gs, char *name, int *value);
int gtc_scope_option_set_boolean (GtcScope *gs, char *name, int value);
int gtc_scope_option_get_double (GtcScope *gs, char *name, double *value);
int gtc_scope_option_set_double (GtcScope *gs, char *name, double value);
int gtc_scope_option_get_string (GtcScope *gs, char *name, char **value);
int gtc_scope_option_set_string (GtcScope *gs, char *name, char *value);

/* observatory control elements */
int gtc_observatory_new (GtcObservatory **gop, GtcModule *gm);
int gtc_observatory_connect (GtcObservatory *go, char *device);
int gtc_observatory_close (GtcObservatory *go);
int gtc_observatory_free (GtcObservatory **gop);
int gtc_observatory_slew_cw_start (GtcObservatory *go);
int gtc_observatory_slew_ccw_start (GtcObservatory *go);
int gtc_observatory_slew_stop (GtcObservatory *go);
int gtc_observatory_skylight_open (GtcObservatory *go);
int gtc_observatory_skylight_close (GtcObservatory *go);
int gtc_observatory_light_enable (GtcObservatory *go, int enable);
int gtc_observatory_nightlight_enable (GtcObservatory *go, int enable);

/* Error message functions */
const char *gtc_get_strerror (int error);

/* Retreive telescope control values */
int gtc_scope_control_get_slewrates (GtcScope *gs, GtcControl **gcp);
int gtc_scope_control_get_trackrates (GtcScope *gs, GtcControl **gcp);
int gtc_scope_control_get_guiderates (GtcScope *gs, GtcControl **gcp);
int gtc_scope_control_get_focus_speeds (GtcScope *gs, GtcControl **gcp);
int gtc_scope_control_get_reticle_flashrates (GtcScope *gs, GtcControl **gcp);
int gtc_scope_control_get_reticle_dutyrates (GtcScope *gs, GtcControl **gcp);


#endif

